// Generated by CoffeeScript 2.1.1
// Import
var Getter, TaskGroup, extendr, typeChecker;

extendr = require('extendr');

typeChecker = require('typechecker');

({TaskGroup} = require('taskgroup'));

Getter = (function() {
  // Getter
  class Getter {
    
    // Constructor
    // Create a new contributors instance
    // opts={githubClientId, githubClientSecret} - also forwarded onto feedr
    constructor(opts = {}) {
      // Prepare
      this.config = {};
      this.entriesMap = {};
      // Extend configuration
      extendr.extend(this.config, {
        log: null,
        onlyLatest: true
      }, opts);
      // Feedr
      this.feedr = require('feedr').create(this.config);
      // Chain
      this;
    }

    // Log
    log(...args) {
      var base;
      if (typeof (base = this.config).log === "function") {
        base.log(...args);
      }
      return this;
    }

    // =================================
    // Add

    // Add an entry to the internal listing and finish preparing it
    // entry = {}
    // return {}
    addEntry(entry) {
      var base, name;
      // Log
      this.log('debug', 'Adding the package:', entry != null ? entry.name : void 0);
      if (!(entry != null ? entry.name : void 0)) {
        // Check
        return null;
      }
      // Update references in database
      if ((base = this.entriesMap)[name = entry.name] == null) {
        base[name] = entry;
      }
      // Return
      return this.entriesMap[entry.name];
    }

    // =================================
    // Format

    // Get the entries
    // return []
    getEntries(entries) {
      var comparator, exists;
      // Log
      this.log('debug', 'Get packages');
      // Prepare
      comparator = function(a, b) {
        var A, B;
        A = a.name.toLowerCase();
        B = b.name.toLowerCase();
        if (A === B) {
          return 0;
        } else if (A < B) {
          return -1;
        } else {
          return 1;
        }
      };
      // Allow the user to pass in their own array or object
      if ((entries != null) === false) {
        entries = this.entriesMap;
      } else {
        // Remove duplicates from array
        if (typeChecker.isArray(entries) === true) {
          exists = {};
          entries = entries.filter(function(repo) {
            var name;
            if (!(repo != null ? repo.name : void 0)) {
              return false;
            }
            if (exists[name = repo.name] == null) {
              exists[name] = 0;
            }
            ++exists[repo.name];
            return exists[repo.name] === 1;
          });
        }
      }
      // Convert objects to arrays
      if (typeChecker.isPlainObject(entries) === true) {
        entries = Object.keys(entries).map((key) => {
          return entries[key];
        });
      }
      // Prepare the result
      entries = entries.sort(comparator);
      // Return
      return entries;
    }

    // =================================
    // Fetch

    // Fetch Package Information from Registry
    // entryName="getpackages"
    // next(err)
    // return @
    requestPackage(entryName, opts = {}, next) {
      var feedOptions, me;
      // Prepare
      me = this;
      // Prepare
      feedOptions = {
        url: `http://registry.npmjs.org/${entryName}`,
        parse: 'json'
      };
      if (this.config.onlyLatest === true) {
        // Only latest?
        feedOptions.url += "/latest";
      }
      // Log
      this.log('debug', 'Requesting package:', entryName, opts, feedOptions);
      // Read the repo's package file
      this.feedr.readFeed(feedOptions, function(err, entry) {
        var addedEntry;
        if (err) {
          // Check
          return next(err, {});
        }
        // Add
        addedEntry = me.addEntry(entry);
        // Return
        return next(null, addedEntry);
      });
      return this;
    }

    // Fetch Packages by Name
    // entryNames=["getpackages"]
    // next(err)
    // return @
    fetchPackagesByNames(entryNames, next) {
      var entries, me, tasks;
      // Prepare
      me = this;
      // Log
      this.log('debug', 'Fetch packages by name:', entryNames);
      // Split it up, github search only supports 79 repos per search it seems...
      // we thought it was due to url length but that doesn't seem to be the case
      // 1732 url with 80 repos fails, 1732 url with 79 repos passes
      entries = [];
      tasks = TaskGroup.create({
        concurrency: 0
      }).done(function(err) {
        var result;
        if (err) {
          // Check
          return next(err, []);
        }
        result = me.getEntries(entries);
        return next(null, result);
      });
      // Add the tasks
      entryNames.forEach(function(entryName) {
        return tasks.addTask(function(complete) {
          return me.requestPackage(entryName, {}, function(err, entry) {
            if (err) {
              return complete(err);
            }
            if (entry) {
              entries.push(entry);
            }
            return complete();
          });
        });
      });
      // Run the tasks
      tasks.run();
      return this;
    }

    // Fetch Packages by Keyword
    // keyword="docpad-plugin"
    // next(err)
    // return @
    fetchPackagesByKeyword(keyword, next) {
      var feedOptions, feedUrl, me;
      // Prepare
      me = this;
      // Prepare feed
      feedUrl = `http://registry.npmjs.org/-/_view/byKeyword?startkey=%5B%22${keyword}%22%5D&endkey=%5B%22${keyword}%22,%7B%7D%5D&group_level=3`;
      feedOptions = {
        url: feedUrl,
        parse: 'json'
      };
      // Log
      this.log('debug', 'Requesting repos from search:', keyword, feedOptions);
      // Read the user's repository feeds
      this.feedr.readFeed(feedOptions, function(err, data) {
        var entryNames, ref;
        if (err) {
          // Check
          return next(err, []);
        }
        if (!(data != null ? (ref = data.rows) != null ? ref.length : void 0 : void 0)) {
          return next(null, []);
        }
        // Add
        entryNames = [];
        data.rows.forEach(function(row) {
          var entryName;
          entryName = row.key[1];
          return entryNames.push(entryName);
        });
        // Forward
        return me.fetchPackagesByNames(entryNames, next);
      });
      return this;
    }

  };

  // Entries
  // Object listing of all the entries mapped by their name
  Getter.prototype.entriesMap = null; // {}

  
  // Config
  Getter.prototype.config = null; // {}

  return Getter;

}).call(this);

// Export
module.exports = {
  create: function(...args) {
    return new Getter(...args);
  }
};
